
#ifndef __DESKTOPMANAGER
#define __DESKTOPMANAGER

#include <windows.h>

#define DM_SETHOOKMSG			"Desktop Manager SetHook Message"
#define DM_GETHOOKMSG			"Desktop Manager GetHook Message"
#define DM_GETXMSG				"Desktop Manager GetX Message"
#define DM_SWITCHTOXMSG			"Desktop Manager SwitchToX Message"
#define DM_MOVEWINTOXMSG		"Desktop Manager MoveWinToX Message"
#define DM_NEWREGSETTINGSMSG	"Desktop Manager NewRegSettings Message"
#define DM_QUITMSG				"Desktop Manager Quit Message"
#define DM_HOOKRETURNMSG		"Desktop Manager HookReturn Message"
#define DM_ENABLESAVERMSG		"Desktop Manager EnableSaver Message"
#define DM_INVOKEPREFSMSG		"Desktop Manager InvokePrefs Message"
#define DM_SENDSPYINFOMSG		"Desktop Manager SendSpyInfo Message"
#define DM_UPDATEMSG			"Desktop Manager Update Message"
#define DM_INTEGRATEMSG			"Desktop Manager Integrate Message"
#define DM_SETSTICKYMSG			"Desktop Manager SetSticky Message"

#define	HOOK_REMOVE			0x0000
#define	HOOK_CLOSING		0x0001
#define	HOOK_PRESWITCH		0x0002
#define	HOOK_POSTSWITCH		0x0004
#define	HOOK_SIZING			0x0008
#define HOOK_IGNORE			0x0020
#define HOOK_PREPREFS		0x0080
#define HOOK_POSTPREFS		0x0100

//Reserved

#define HOOK_SPYINFO		0x8000

#define MODULEMUTEX "Desktop Manager Module Busy"

/*
	Plugin API

	Idea is simple, just send a message to the desktop manager window
	with one of the following commands:

	SendMessage/PostMessage(HWND win, UINT msg, WPARAM wParam, LPARAM lParam);

	DM_SETHOOK:
		wParam is the HWND that will be getting DM_HOOKRETURNMSG messages.
				Note: unrelated usage to same name below.
		
		lParam one or more of the following:
				HOOK_REMOVE      removes hook
				HOOK_CLOSING	 message will be sent when DM is about to exit 
				HOOK_PRESWITCH   message will be sent before DM switches desktops
				HOOK_POSTSWITCH  message will be sent after DM switches desktops
				HOOK_SIZING      message will be sent for each DM WM_SIZING msg.
				HOOK_IGNORE		 HWND will be ignored by DM
				HOOK_TRANSPARENT HWND will be sent Listview messages for 
								 transparent icon text
				HOOK_PREPREFS    message will be sent before DM changes it's settings.
				HOOK_POSTPREFS   message will be sent after DM changes it's settings.
		
		return value is 1 for success, 0 for failure

		HWND will be sent a DM_HOOKRETURNMSG except for HOOK_IGNORE:
		wParam will be one of above HOOK_* values 
		lParam will be HOOK_* specific data
			HOOK_PRE/POSTSWITCH lParam = old << 16 + new.
	
	DM_GETHOOK:
		wParam HWND you want hook flags for
		return value is the value passed in the DM_SETHOOK message for the HWND.
	
	DM_GETX:
		returns an integer representing the current desktop
	
	DM_SWITCHTOX:
		wParam is an index from 0..N of the desktop to switch to. 
		Use Desktops in the DM registry to get N.
	
	DM_MOVEWINTOX:
		wParam is the window, lParam is the desktop to move the window to.
	
	DM_NEWREGSETTINGS:
		Forces the program to read in current registry settings.

	DM_HOOKRETURNMSG
		*Not used*
	
	DM_ENABLESAVER
		wParam speedscale*256+speed to enable, 0 to disable.
		lParam HWND
		call from a screensaver as an option.
		ie: speed ranges from 1..12, current speed is 2:
			WM_CREATE:
				PostMessage(win, DM_ENABLESAVER, (12 << 8)+2. hWin);
			WM_DESTROY:
				PostMessage(win, DM_ENABLESAVER, 0. hWin);
	
	DM_INVOKEPREFS
		Brings up the DM prefs window
		wParam is parent window to use, can be NULL.
	
	DM_INTERGRATEMSG
		
		wParam is a window handle.
		LOWORD(lParam) is an integer describing placement within the DM strip, 
		positive numbers mean above DM, negative numbers mean below DM. Numbers are sorted for placement, 
		a module with a -1 position will be just below the DM window, one with a -2 will just be below that.
		Modules with the same position number will be in first come, firstserve order.
		HIWORD(lParam) is the height of the requested placement, -1 means use as much as possible.
		
		DM handles resizing of integrated modules, modules should use WM_WINDOWPOSCHANGED to
		handle new window sizes and positions.

	DM_UPDATEMSG
		DM checks it's windows and redraws the current desktop to reflect window positions, etc.

	DM_SETSTICKYMSG
		wParam is a window.
		lParam is true to make sticky, false to make unsticky
		
	Ex:

	DWORD integratemsg = 0;
	HWND mywindow;
	DWORD mywindowheight = 33;
	DWORD mywindowpos = 2;

	HWND win = FindWindow("Desktop Manager Class", "Desktop Manager");
	integratemsg = RegisterWindowMessage(DM_INTEGRATEMSG);

	if (win && integratemsg)
		SendMessage(win, integratemsg, (WPARAM) mywindow, (mywindowheight << 16) | LOWORD(mywindowpos)); 

*/

#endif
