/*
Example Clock Module

Full module program to display the current time in the DM strip.

Basic use of DM_INTEGRATEMSG is demonstrated.

Source is free to use.
*/

#include <desktop.h>
#include <commctrl.h>
#include <stdio.h>
#include "resource.h"

#define CLASSNAME "DM Clock Class"
#define APPNAME "DM Clock"

DWORD newregsettingsmsg = 0;
DWORD getxmsg = 0;
DWORD hookretmsg = 0;
DWORD sethookmsg = 0;
DWORD invokeprefsmsg = 0;
DWORD integratemsg = 0;

HINSTANCE mainapp;
HWND mainwin;
HWND dmwin;
HWND tooltipwin;
long Desktop;

long backcol = RGB(160,199,160);
long clockcol = RGB(20,60,20);
long shadcol = -1;
long noampm = false;

char datestring[MAX_PATH]; //For the tooltip

bool LoadSettings(long flag);

void ResizeTooltip(RECT z) 
{ 
	TOOLINFO ti;
	int d;

    if (!tooltipwin) 
        return; 
 
	ti.cbSize = sizeof(TOOLINFO);
	ti.hwnd = mainwin;
	ti.uId = (UINT) 1; 
	ti.rect = z;
 
	SendMessage(tooltipwin, TTM_NEWTOOLRECT, 0, (LPARAM) &ti);
}  
 
HWND CreateToolTip(RECT z) 
{ 
    HWND win;    
	int d;
    TOOLINFO ti;

	win = CreateWindowEx(WS_EX_TOPMOST,TOOLTIPS_CLASS, (LPSTR) NULL, TTS_ALWAYSTIP, 
        CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, 
        NULL, (HMENU) NULL, mainapp, NULL); 
 
    if (win == NULL) 
        return NULL; 
 
	ti.cbSize = sizeof(TOOLINFO); 
       ti.uFlags = TTF_SUBCLASS|TTF_CENTERTIP; 
	ti.hwnd = mainwin; 
	ti.hinst = mainapp; 
	ti.uId = (UINT) 1; 
	ti.lpszText = (LPSTR) LPSTR_TEXTCALLBACK; 

	ti.rect = z;
		
	if (!SendMessage(win, TTM_ADDTOOL, 0, (LPARAM) &ti)) 
          return NULL; 

	PostMessage(win, TTM_ACTIVATE, true, 0);

    return win; 
}

char *Months[] = {
	"January",
	"February",
	"March",
	"April",
	"May",
	"June",
	"July",
	"August",
	"September",
	"October",
	"November",
	"December"
};

char *Days[] = {
	"Sunday",
	"Monday", 
	"Tuesday",
	"Wednesday",
	"Thursday",
	"Friday",
	"Saturday"
};

char *Fun[] = {
	" (Thank God!)",
	" (Whoopie!)",
	" (Yes!!)",
	" (Party!!)",
};

void DoRender(HDC hdc, RECT *z)
{
	char buffer[MAX_PATH];
	SYSTEMTIME blah;
	HBRUSH brush;

	GetLocalTime(&blah);

	FillRect(hdc, z, brush = CreateSolidBrush(backcol));
	DeleteObject(brush);

	DrawEdge(hdc, z, EDGE_BUMP, BF_RECT); 

	sprintf(datestring, "%s%s, %s %d%s, %.4d", Days[blah.wDayOfWeek], (blah.wDayOfWeek == 5) ? Fun[rand() & 3] : "", Months[blah.wMonth - 1], blah.wDay, ((blah.wDay % 10) == 3 && (blah.wDay != 13)) ? "rd" : "th", blah.wYear);
	
	if (noampm)
		sprintf(buffer, "%2d:%.2d", blah.wHour, blah.wMinute);
	else
		sprintf(buffer, "%2d:%.2d %s", blah.wHour == 0 ? 12 : blah.wHour > 12 ? blah.wHour - 12 : blah.wHour, blah.wMinute, blah.wHour >= 12 ? "PM" : "AM");

	SetBkMode(hdc, TRANSPARENT);

	if (shadcol >= 0)
	{
		SetTextColor(hdc, shadcol);

		DrawText(hdc, buffer, strlen(buffer), z, DT_SINGLELINE|DT_VCENTER|DT_CENTER);

		OffsetRect(z, -1, -1);
	}
	SetTextColor(hdc, clockcol);
	DrawText(hdc, buffer, strlen(buffer), z, DT_SINGLELINE|DT_VCENTER|DT_CENTER);
	ValidateRect(hdc, z);
}

void DoNotify(HWND win, UINT msg, WPARAM wParam, LPARAM lParam )
{
	LPTOOLTIPTEXT ttt;
	int len;

	switch (((LPNMHDR) lParam)->code)
	{
	case TTN_NEEDTEXT:
		ttt = (LPTOOLTIPTEXT) lParam;

		if (wParam == 1)
		{
			len = strlen(datestring);
			strcpy(ttt->szText, datestring);
		}
		break;

	case TTN_SHOW:
		SetWindowPos(tooltipwin, HWND_TOPMOST, 0, 0,0,0, SWP_NOMOVE|SWP_NOSIZE);
		break;
	}
}

LRESULT CALLBACK DoWindows(HWND win, UINT msg, WPARAM wParam, LPARAM lParam)
{
	long i, ret = 0;
	bool quit = false;
	PAINTSTRUCT paint;
	HDC hdc;
	LPWINDOWPOS wp;
	RECT z;

	if (hookretmsg && msg == hookretmsg)
	{
		switch (wParam)
		{
		case HOOK_CLOSING:
			quit = true;
			break;

		case HOOK_PRESWITCH:
			Desktop = LOWORD(lParam);
			break;

		case HOOK_POSTPREFS:
			break;
		}
	}
	else
	{
		if (newregsettingsmsg && msg == newregsettingsmsg)
		{
			LoadSettings(wParam != 1 ? wParam : 0);
			return false;
		}

		switch( msg )
		{
		case WM_NULL:
			break;

		case WM_WINDOWPOSCHANGED: //Redraw the window and resize the hit box for the tooltip, DM has moved us

			GetClientRect(win, &z);
			InvalidateRect(win, &z, true);

			if (tooltipwin)
				ResizeTooltip(z);
			break;

		case WM_ERASEBKGND: //No need to erase, WM_PAINT fills everything.
			ret = 1;
			break;

		case WM_NOTIFY:
			DoNotify(win, msg, wParam, lParam);
			break;

		case WM_NCPAINT:
			break;

		case WM_TIMER:

			if (wParam == 1)
			{
				GetClientRect(win, &z);
				InvalidateRect(win, &z, true);
				UpdateWindow(win);
			}
			break;

		case WM_PAINT:
			if (hdc = BeginPaint(win, &paint))
			{
				GetClientRect(win, &z);
				DoRender(hdc, &z);
				EndPaint(win, &paint);
			}
			break;

		case WM_DESTROY:
			quit = true;
			break;
		
		default:
			ret = DefWindowProc(win, msg, wParam, lParam );
			break;
		}
	}

	if (quit)
		PostQuitMessage(0);

   return ret;
}

bool LoadSettings(long flag)
{
	WNDCLASSEX wc;
	RECT z;
	HWND temp;

	if (flag == 1)
	{
		wc.cbSize = sizeof(WNDCLASSEX);
		wc.lpszClassName = CLASSNAME;
		wc.lpfnWndProc = DoWindows;
		wc.style = CS_OWNDC;
		wc.hInstance = mainapp;
		wc.hIcon = LoadIcon(mainapp, MAKEINTRESOURCE(IDI_ICON));
		wc.hCursor = NULL;
		wc.hbrBackground =  NULL;
		wc.lpszMenuName = NULL;
		wc.cbClsExtra = 0;
		wc.cbWndExtra = 0;
		wc.hIconSm = NULL;

		RegisterClassEx(&wc);

		if (mainwin = CreateWindowEx(WS_EX_TOOLWINDOW, CLASSNAME, APPNAME, WS_POPUP|WS_VISIBLE,
									0,0,70,22,
									NULL, NULL, mainapp, NULL))
		{
			if (dmwin = FindWindow("Desktop Manager Class", "Desktop Manager"))
			{
				SendMessage(dmwin, sethookmsg, (long) mainwin, HOOK_IGNORE|HOOK_CLOSING); //Have DM say when it is exiting and to ignore our window
				SendMessage(dmwin, integratemsg, (long) mainwin, (22 << 16) | LOWORD(-1));
				GetClientRect(mainwin, &z);
				tooltipwin = CreateToolTip(z);
				SetTimer(mainwin, 1, 60 * 1000, NULL); //Send a WM_TIMER message every minute to update the clock
				SetCursor(LoadCursor(NULL, IDC_ARROW)); //Reset the cursor for some reason.
			}
		}
	}
	else
	{
		temp = dmwin;

		if ((dmwin = FindWindow("Desktop Manager Class", "Desktop Manager")) && temp != dmwin)
		{
			SendMessage(dmwin, sethookmsg, (long) mainwin, HOOK_IGNORE|HOOK_CLOSING);
			SendMessage(dmwin, integratemsg, (long) mainwin, (22 << 16) | LOWORD(-1));
		}
	}
	return dmwin ? true : false;
}

void CleanUp(void)
{
	if (dmwin = FindWindow("Desktop Manager Class", "Desktop Manager"))
	{
		SendMessage(dmwin, integratemsg, (long) mainwin, 0);
		SendMessage(dmwin, sethookmsg, (long) mainwin, HOOK_REMOVE);
	}

	KillTimer(mainwin, 1);
	DestroyWindow(mainwin);
}

int WINAPI WinMain(HINSTANCE app, HINSTANCE prev_app, LPSTR args, int win_show) 
{
	MSG msg;
	HANDLE running;
	HWND win;

	//Register DM Window Messages

	newregsettingsmsg = RegisterWindowMessage(DM_NEWREGSETTINGSMSG);
	getxmsg =			RegisterWindowMessage(DM_GETXMSG);
	hookretmsg =		RegisterWindowMessage(DM_HOOKRETURNMSG);
	sethookmsg =		RegisterWindowMessage(DM_SETHOOKMSG);
	invokeprefsmsg =	RegisterWindowMessage(DM_INVOKEPREFSMSG);
	integratemsg =		RegisterWindowMessage(DM_INTEGRATEMSG);
	
	running = CreateMutex(NULL, true, APPNAME); 

	if (GetLastError() != 0) //Already Running
	{
		if (win = FindWindow(CLASSNAME, APPNAME))
			SendMessage(win, newregsettingsmsg, 0, 0);
		return 0;
	}
	
	InitCommonControls();

	mainapp = app;

	if (LoadSettings(1))
	{
		while (GetMessage(&msg, NULL, 0, 0 ) > 0)
		{
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
	}

	CleanUp();
	
	ReleaseMutex(running);
	CloseHandle(running);

	return msg.wParam;
}
