/************
Example Prefs Module

Add a Tab to the DM prefs for editing hotkeys. Select the Desktop and edit the hotkey.

Source is free to use.

*************/

#include "desktop.h"
#include <commctrl.h>
#include "resource.h"
#include <stdio.h>
#include "strtmenu.h"

HWND maindialog = NULL;

char RootKey[] = "Software\\Desktop Manager"; //Registry path to main DM key

int Desktop = 0; //Current Desktop
int desktopnum;

typedef struct desktop_hotkey_struct {
	DWORD key;
	bool defined;
	bool changed;
} HKey, *LPHKey;

HKey DesktopHotKeys[9]; //Allocate 9, max num of desktops

DWORD hotkeymodifier = 0;

DWORD HookRetMsg = 0;
DWORD NewRegSettingsMsg = 0;
DWORD GetXMsg = 0;
DWORD SetHookMsg = 0;

bool RegisterDMsgs()
{
	//Register Desktop Manager interface messages

	if ((HookRetMsg = RegisterWindowMessage(DM_HOOKRETURNMSG)) &&
		(NewRegSettingsMsg = RegisterWindowMessage(DM_NEWREGSETTINGSMSG)) &&
		(GetXMsg = RegisterWindowMessage(DM_GETXMSG)) &&
		(SetHookMsg = RegisterWindowMessage(DM_SETHOOKMSG)))
		return true;
	else
		return false;  //Called From DM yet no messages, Hmm, old version of DM?
}

void SetDialogText(HWND win)
{
	char buff[8];

	sprintf(buff, "%d", Desktop);
	
	if (win = GetDlgItem(win, IDC_DESKTOP))
		SetWindowText(win, buff);
}

void SetCurrentKey(HWND win)
{
	DWORD key = 0, modifier = 0;
	
	if (DesktopHotKeys[Desktop].defined)
	{
		key = DesktopHotKeys[Desktop].key;
		modifier = hotkeymodifier;
	}
	
	SendDlgItemMessage(win, IDC_DESKHOTKEY, HKM_SETHOTKEY, MAKEWORD(key, modifier), 0); 
}

void LoadSettings(HWND win)
{
	int i;
	HWND temp;
	char path[MAX_PATH];

	if (temp = FindWindow("Desktop Manager Class", "Desktop Manager"))
	{
		Desktop = SendMessage(temp, GetXMsg, 0, 0);
		SendMessage(temp, SetHookMsg, (WPARAM) win, HOOK_PRESWITCH);
	}

	SetDialogText(win);

	if (!LoadReg(HKCU, RootKey, "Desktops", &desktopnum, 4)) //Get the total number of desktops, max 9
		desktopnum = 4;
	
	if (!LoadReg(HKCU, RootKey, "HotkeyModifier", &hotkeymodifier, 4))
		hotkeymodifier = MOD_CONTROL;

	for (i = 0; i < desktopnum; i++)
	{
		sprintf(path, "%s\\%d", RootKey, i);
		DesktopHotKeys[i].changed = false;
		DesktopHotKeys[i].defined = LoadReg(HKCU, path, "HotKey", &DesktopHotKeys[i].key, 4) ? true : false;
	}

	SetCurrentKey(win);
}

static bool beenupdated = false;

void SaveSettings(HWND win)
{
	char path[MAX_PATH];
	HWND temp;
	int i;

	if (beenupdated)
		return;

	beenupdated = true;

	for (i = 0; i < desktopnum; i++)
	{
		if (DesktopHotKeys[i].changed)
		{
			sprintf(path, "%s\\%d", RootKey, i);
			SaveReg(HKCU, path, "HotKey", &DesktopHotKeys[i].key, REG_DWORD, 4);
		}
	}

	SaveReg(HKCU, RootKey, "HotkeyModifier", &hotkeymodifier, REG_DWORD, 4);

	if (temp = FindWindow("Desktop Manager Class", "Desktop Manager"))
	{
		SendMessage(temp, NewRegSettingsMsg, 0, 0); //Tell DM to reload it's prefs.
	}
}

BOOL DoCommand(HWND win, WPARAM wParam, LPARAM lParam)
{
	bool ret = false;
	DWORD d;

	switch (LOWORD(wParam))
	{
	case IDC_DESKHOTKEY: //Hotkey changed, save it.

		d = SendDlgItemMessage(win, IDC_DESKHOTKEY, HKM_GETHOTKEY, 0, 0);
		
		hotkeymodifier = d >> 8;
		DesktopHotKeys[Desktop].key = d & 255;
		DesktopHotKeys[Desktop].defined = true;
		DesktopHotKeys[Desktop].changed = true;
		ret = true;
		break;
	}

	return ret;
}

DWORD CALLBACK DoDialog(HWND win, UINT msg, WPARAM wParam, LPARAM lParam)
{
	DWORD ret = false, apply = false;

	//Add default DM background, colour controls, do custom groupbox controls, etc.
	ret = DoDefaultPrefsDialog(win, msg, wParam, lParam, "Ex.", "DM.CHM", NULL); 
	
	if (HookRetMsg && HookRetMsg == msg) //DM Notification message sent upon application request
	{
		switch (wParam)
		{
		case HOOK_PRESWITCH: //Sent just before DM changes desktops
			Desktop = lParam & 255;
			SetDialogText(win);
			SetCurrentKey(win);	
			break;
		}
	}

	switch (msg)
	{
	case WM_INITDIALOG:
		ret = true;
		break;

	case WM_COMMAND:
		apply = DoCommand(win, wParam, lParam); 
		break;

	case WM_NOTIFY:
		
		#define header ((NMHDR FAR *) lParam)

		switch(header->code)
		{
		case PSN_SETACTIVE:

			maindialog = header->hwndFrom;
			LoadSettings(win);

		case PSN_KILLACTIVE:
			
			SetWindowLong(win, DWL_MSGRESULT, false);
			ret = true;
			break;

		case PSN_APPLY:
			
			SaveSettings(win);

			SetWindowLong(win, DWL_MSGRESULT, PSNRET_NOERROR);
			ret = true;
			break;

		case PSN_RESET:

			ReleaseStartMenuLib();
			break;

		default:
			break;
		}
		break;
	}
	
	if (apply)
	{
		beenupdated = false;
		PropSheet_Changed(maindialog, win); 
	}

	return ret;
}

//Entry function called by DM to add this page to it's prefs

BOOL CALLBACK ExtensionPropSheetPageProc( LPVOID lpv, LPFNADDPROPSHEETPAGE func, LPARAM lParam)
{
	BOOL ret = false;
	PROPSHEETPAGE psp;

	if (LoadStartMenuLib())
	{
		psp.dwSize = sizeof(PROPSHEETPAGE);
		psp.dwFlags = PSP_USETITLE|PSP_HASHELP;
		psp.hInstance = (HINSTANCE) GetModuleHandle("exprefs.dll");
		psp.pszTemplate = MAKEINTRESOURCE(IDD_PREFS);
		psp.pfnDlgProc = (DLGPROC) DoDialog;
		psp.pszTitle = "Ex.";

		if (psp.hInstance && RegisterDMsgs())
			ret = func((HPROPSHEETPAGE) CreatePropertySheetPage(&psp), 0);
		else
			ReleaseStartMenuLib();
	}

	return ret;
}
